Module types

Module types 

Structs

Achievement
Administrator
Ban
Highscore
Player
Contains the data of a connected Player
Sync
An object that’s being synced between players
SyncIter
A return value from an iterator containing data about a Sync from another player

Enums

AdminAction
DataUpdate
Data updates that have been triggered from the server.
LoginCode
The return success value of a login attempt
OptionalValue
PlayerRequest
The target it should request something from/to
PlayerRequestCode
The target it should request something from/to
RegistrationCode
The return success value of a registration attempt
SyncEvent
SyncType
The target syncronization type
Value
A structure that represents a dynamic value.

Type Aliases

FetchBdbServerUpdate
PlayerVariableServerUpdate
SyncVariableServerUpdate