Structs
- Achievement
- Administrator
- Ban
- Highscore
- Player
- Contains the data of a connected Player
- Sync
- An object that’s being synced between players
- Sync
Iter - A return value from an iterator containing data about a Sync from another player
Enums
- Admin
Action - Data
Update - Data updates that have been triggered from the server.
- Login
Code - The return success value of a login attempt
- Optional
Value - Player
Request - The target it should request something from/to
- Player
Request Code - The target it should request something from/to
- Registration
Code - The return success value of a registration attempt
- Sync
Event - Sync
Type - The target syncronization type
- Value
- A structure that represents a dynamic value.